import { Cache, IncrementalFileValue } from './interfaces.js';
import { Config } from './Config.js';
export declare class Incremental implements Cache {
    private readonly _cacheFolder;
    private _filePaths;
    readonly config: Config;
    get cacheFolder(): string;
    get filePaths(): Record<string, IncrementalFileValue>;
    set filePaths(value: Record<string, IncrementalFileValue>);
    /**
     * Creates an instance of Incremental.
     */
    constructor(config?: Config);
    /**
     * Read config (.gzipperconfig).
     */
    readConfig(): Promise<void>;
    /**
     * update config (.gzipperconfig).
     */
    updateConfig(): Promise<void>;
    /**
     * Create cache folder (.gzipper).
     */
    initCacheFolder(): Promise<void>;
    /**
     * Returns file incremental info and save checksum and options to `filePath` (if file is changed or newly created).
     */
    setFile(target: string, checksum: string, compressionType: string, compressOptions: IncrementalFileValue['revisions'][number]['options']): {
        isChanged: boolean;
        fileId: string;
    };
    /**
     * Returns file checksum.
     */
    getFileChecksum(target: string): Promise<string>;
    /**
     * Purge cache folder.
     */
    cachePurge(): Promise<void>;
    /**
     * Returns cache size.
     */
    cacheSize(folderPath?: string, size?: number): Promise<number>;
}
